@@ -106,6 +106,22 @@ class MarketStats:
 
         return [past_buy_target, past_sell_target]
 
+    def calculate_past_trend(self, past_targets, mean):
+        target_distances = np.abs(np.asarray(past_targets) - mean)
+        target_distances_normalized = target_distances/np.sum(target_distances)
+        past_trend = 1.0-target_distances_normalized
+        return past_trend
+
+    def calculate_present_trend(self, present_targets, mean):
+        target_distances = np.abs(np.asarray(present_targets) - mean)
+        target_distances_normalized = target_distances/np.sum(target_distances)
+        present_trend = 1.0-target_distances_normalized
+        return present_trend
+
+    def calculate_mean_trend(self, past_trend, present_trend):
+        mean_trend = (past_trend + present_trend)/2
+        return mean_trend
+
     def price_weighted(self, units=1):
         tradesDF = self.trades_df()
         tradesDF = tradesDF[tradesDF['acc_vol']<units]
@@ -113,20 +129,16 @@ class MarketStats:
         price_weighted = np.sum(tradesDF['price']*tradesDF['vol_rate'])
         return price_weighted
     
-    def present_trend(self, mean, units=1):
-        target_distances = np.abs(np.asarray(self.present_targets()) - mean)
-        target_distances_normalized = target_distances/np.sum(target_distances)
-        present_trend = 1.0-target_distances_normalized
+    def present_trend(self, mean):
+        present_trend = self.calculate_present_trend(self.present_targets(), mean)
         return present_trend
     
     def past_trend(self, mean, units=1):
-        target_distances = np.abs(np.asarray(self.past_targets(units)) - mean)
-        target_distances_normalized = target_distances/np.sum(target_distances)
-        past_trend = 1.0-target_distances_normalized
+	past_trend = self.calculate_past_trend(self.past_targets(units=units), mean)
         return past_trend
     
     def mean_trend(self, mean, past_units=1):
-        mean_trend = (self.past_trend(mean, past_units)+self.present_trend(mean))/2
+        mean_trend = self.calculate_mean_trend(self.past_trend(mean, past_units), self.present_trend(mean))
         return mean_trend
     
     def ticker_stats(self):
@@ -149,22 +161,47 @@ class MarketStats:
         last_orderbook = self.last_orderbook()
         volume_trend = self.volume_trend()
         orderbook_timestamp = last_orderbook[0]
-        past_trend_unit1 = self.past_trend(ticker_stats['bsm'], units=1)
-        past_trend_unit2 = self.past_trend(ticker_stats['bsm'], units=2)
-        past_trend_unit3 = self.past_trend(ticker_stats['bsm'], units=3)
-        present_trend = self.present_trend(ticker_stats['bsm'])
-        mean_trend = self.mean_trend(ticker_stats['bsm'])
+
+        past_targets_unit1 = self.past_targets(units=1)
+        past_targets_unit2 = self.past_targets(units=2)
+        past_targets_unit3 = self.past_targets(units=3)
+        past_targets_unit4 = self.past_targets(units=4)
+        past_targets_unit5 = self.past_targets(units=5)
+        past_trend_unit1 = self.calculate_past_trend(past_targets_unit1, ticker_stats['bsm'])
+        past_trend_unit2 = self.calculate_past_trend(past_targets_unit2, ticker_stats['bsm'])
+        past_trend_unit3 = self.calculate_past_trend(past_targets_unit3, ticker_stats['bsm'])
+        past_trend_unit4 = self.calculate_past_trend(past_targets_unit4, ticker_stats['bsm'])
+        past_trend_unit5 = self.calculate_past_trend(past_targets_unit5, ticker_stats['bsm'])
+
+	present_targets = self.present_targets()
+        present_trend = self.calculate_present_trend(present_targets, ticker_stats['bsm'])
+
         all_stats = {}
         all_stats.update(ticker_stats)
         all_stats.update(dict(zip(['past_buy_trend_unit1', 'past_sell_trend_unit1'], past_trend_unit1)))
         all_stats.update(dict(zip(['past_buy_trend_unit2', 'past_sell_trend_unit2'], past_trend_unit2)))
         all_stats.update(dict(zip(['past_buy_trend_unit3', 'past_sell_trend_unit3'], past_trend_unit3)))
-        all_stats.update(dict(zip(['present_buy_trend', 'present_sell_trend'],present_trend)))
-        all_stats.update(dict(zip(['mean_buy_trend', 'mean_sell_trend'],mean_trend)))
-        all_stats.update(dict(zip(['volume_buy_trend', 'volume_sell_trend'],volume_trend)))
+        all_stats.update(dict(zip(['past_buy_trend_unit4', 'past_sell_trend_unit4'], past_trend_unit4)))
+        all_stats.update(dict(zip(['past_buy_trend_unit5', 'past_sell_trend_unit5'], past_trend_unit5)))
+        all_stats.update(dict(zip(['present_buy_trend', 'present_sell_trend'], present_trend)))
+        all_stats.update(dict(zip(['mean_buy_trend', 'mean_sell_trend'], self.mean_trend(past_trend_unit1, present_trend))))
+        all_stats.update(dict(zip(['mean_buy_trend_unit1', 'mean_sell_trend_unit1'], self.mean_trend(past_trend_unit1, present_trend))))
+        all_stats.update(dict(zip(['mean_buy_trend_unit2', 'mean_sell_trend_unit2'], self.mean_trend(past_trend_unit2, present_trend))))
+        all_stats.update(dict(zip(['mean_buy_trend_unit3', 'mean_sell_trend_unit3'], self.mean_trend(past_trend_unit3, present_trend))))
+        all_stats.update(dict(zip(['mean_buy_trend_unit4', 'mean_sell_trend_unit4'], self.mean_trend(past_trend_unit4, present_trend))))
+        all_stats.update(dict(zip(['mean_buy_trend_unit5', 'mean_sell_trend_unit5'], self.mean_trend(past_trend_unit5, present_trend))))
+        all_stats.update(dict(zip(['past_buy_target_unit1', 'past_sell_target_unit1'], past_targets_unit1)
+        all_stats.update(dict(zip(['past_buy_target_unit2', 'past_sell_target_unit2'], past_targets_unit2)
+        all_stats.update(dict(zip(['past_buy_target_unit3', 'past_sell_target_unit3'], past_targets_unit3)
+        all_stats.update(dict(zip(['past_buy_target_unit4', 'past_sell_target_unit4'], past_targets_unit4)
+        all_stats.update(dict(zip(['past_buy_target_unit5', 'past_sell_target_unit5'], past_targets_unit5)
+        all_stats.update(dict(zip(['present_buy_target', 'present_buy_target'], present_targets)
+        all_stats.update(dict(zip(['volume_buy_trend', 'volume_sell_trend'], volume_trend)))
         all_stats.update({"orderbook_timestamp": orderbook_timestamp})
         all_stats.update({"price_weighted_unit1": self.price_weighted(units=1)})
         all_stats.update({"price_weighted_unit2": self.price_weighted(units=2)})
         all_stats.update({"price_weighted_unit3": self.price_weighted(units=3)})
+        all_stats.update({"price_weighted_unit4": self.price_weighted(units=4)})
+        all_stats.update({"price_weighted_unit5": self.price_weighted(units=5)})
         return all_stats
 
